//
// DLL stuff
//
// by Denis Nechitailov, UU9JDR, 2002
//
// Based on demodulator.c by Tomi Manninen, OH2BNS
// 

//------------------------------------------------------------------------------

#include <stdlib.h>
#include <string.h>

#include "modem.h"
#include "complex.h"
#include "modemconfig.h"
#include "fec.h"
#include "filter.h"
#include "newqpskrx.h"
#include "tbl.h"

#include "unistd.h" 

#define SAMPLERATE(x)	((float)(x)*SymbolLen/DataCarriers/SymbolBits)

//------------------------------------------------------------------------------

typedef void (* SetDcdProc)(void* procparam, int dcd);
typedef void (* RxPktProc)(void* procparam, const unsigned char* pkt, unsigned len);

// Outer structure

struct RxState 
{
	complex csamples[SymbolLen];
	int sptr;

	complex rbuf[SymbolLen / 2];
	int rptr;

	SetDcdProc dcdproc;
	RxPktProc pktproc; 
	void* procparam;

	struct rxstate s; // have rxstate inside
};

//------------------------------------------------------------------------------
//
// Allocate memory and init RxState structure
//

_declspec(dllexport) void *demodconfig(unsigned int *samplerate, const char *params[])
{
	struct RxState *S; 
	struct rxstate *s;

	if ((S = calloc(1, sizeof(struct RxState))) == NULL)
	{
		logprintf(MLOG_FATAL, "out of memory");
		return NULL;
	}

	s = &(S->s);

	s->chan = (struct modemchannel *)S;

	if (params[0]) {
		s->bps = strtoul(params[0], NULL, 0);
		if (s->bps < 1000)
			s->bps = 1000;
		if (s->bps > 5000)
			s->bps = 5000;
	} else
		s->bps = 2500;
	if (params[1]) {
		s->fec.inlv = strtoul(params[1], NULL, 0);
		if (s->fec.inlv < 0)
			s->fec.inlv = 0;
		if (s->fec.inlv > 16)
			s->fec.inlv = 16;
	} else
		s->fec.inlv = 8;
	if (params[2]) {
		s->fec.feclevel = strtoul(params[2], NULL, 0);
		if (s->fec.feclevel < 0)
			s->fec.feclevel = 0;
		if (s->fec.feclevel > 3)
			s->fec.feclevel = 3;
	} else
		s->fec.feclevel = 3;
	if (params[3]) {
		s->mintune = strtoul(params[3], NULL, 0);
		if (s->mintune < 0)
			s->mintune = 0;
		if (s->mintune > 32)
			s->mintune = 32;
	} else
		s->mintune = 16;
	if (params[4]) {
		s->minsync = strtoul(params[4], NULL, 0);
		if (s->minsync < 8)
			s->minsync = 8;
		if (s->minsync > 32)
			s->minsync = 32;
	} else
		s->mintune = 16;
	*samplerate = (int) (3.0 * SAMPLERATE(s->bps) + 0.5);
	return S;
}

//------------------------------------------------------------------------------
//
// Remove RxState structure
//

_declspec(dllexport) void demodkill(void *state)
{
	struct RxState *S = (struct RxState *)state;
	// deinit???

	free(S);
}

//------------------------------------------------------------------------------
//
// Init the demodulator
//

_declspec(dllexport) void demodinit(void *state, unsigned int samplerate, unsigned int *bitrate)
{
	struct RxState *S = (struct RxState *)state;
	struct rxstate *s = &(S->s);
	float rate, f1, f2;

	s->srate = SAMPLERATE(s->bps);
	rate = SAMPLERATE(s->bps) / samplerate;
	f1 = 0.1 * rate;
	f2 = 0.9 * rate;
	init_tbl();
	init_fec(&s->fec);
	init_filter(&s->filt, rate, f1, f2);
	init_newqpskrx(s);
	*bitrate = s->bps;
}

//------------------------------------------------------------------------------

_declspec(dllexport) void demoddemodulate(void *state, short* samples, int numsamp)
{	
	struct RxState *S = (struct RxState *)state;
	struct rxstate *s = &(S->s);
	int n,m;

	for (n=0; n<numsamp; n++)
	{
		S->csamples[S->sptr].re = samples[n] * (1.0 / 32768.0);
		S->csamples[S->sptr].im = S->csamples[S->sptr].re;

		S->sptr++;
		if (S->sptr >= SymbolLen)
		{	
			S->sptr = 0;

			s->buflen = filter(&s->filt, S->csamples, s->rxbuf);

			for (m=0; m<s->buflen; m++)
			{
				if (s->skip>0)
				{
					s->skip--;
					continue;
				}
				else
				if (s<0) // bug?
					s = 0; 
				
				S->rbuf[S->rptr] = s->rxbuf[m];
				S->rptr ++;
				if (S->rptr>=SymbolLen/2)
				{
					newqpskrx(s, S->rbuf);
					S->rptr = 0;
				}
			}
		}
	}
}

//------------------------------------------------------------------------------
//
// Set demodulator callback functions
//

_declspec(dllexport) void demodsetcallbacks(void *state, void(*dcdproc)(void*, int) , void(*pktproc)(void*, const unsigned char*, unsigned), void* procparam)
{
	struct RxState *S = (struct RxState *)state;

	S->dcdproc = dcdproc;
	S->pktproc = pktproc;
	S->procparam = procparam;
}

//------------------------------------------------------------------------------
//
// Call the callback function to set DCD
//

void pktsetdcd(struct modemchannel *chan, int dcd)
{
	struct RxState *S = (struct RxState *)chan;
    (*S->dcdproc)(S->procparam, dcd);
	
}

//------------------------------------------------------------------------------
//
// Call the callback function to put the next received byte
//

void pktput(struct modemchannel *chan, const unsigned char *data, unsigned int len)
{
	struct RxState *S = (struct RxState *)chan;
    (*S->pktproc)(S->procparam, data, len);
}

//------------------------------------------------------------------------------
